<?php

    if ( !defined( 'ABSPATH' ) ) exit;

    /**
     * JSportnaptar898_cache
     * 
     * Gyorsítótár kezelés
     * 
     * @package Sportnaptár plugin  
     * @author InfoScope Kft.
     * @copyright Minden jog fenntartva!
     * @version 2020
     * @access public
     */
    class JSportnaptar898_cache extends stdClass {

        public $ID = '';            // Plugin slug a nyelvi fájlokhoz
        public $path = null;        // cache útvonal

        private $timeout = 6;       // Elavuláshoz órák száma
        private $errors = array();  // Hibabejegyzések tömbje
        
        /**
         * JSportnaptar898_cache::__construct()
         * 
         * @param integer $timeout Elavuláshoz órák száma
         * @return void
         */
        public function __construct($timeout = 6) {
            
            if (!empty($timeout)) { $this->timeout = $timeout; }
            
            $this->path = plugin_dir_path(dirname(__file__)) . 'cache' .DIRECTORY_SEPARATOR;
            if (!is_dir($this->path)) {
                mkdir($this->path, 0775);
                file_put_contents($this->path . 'index.html', '<html><body bgcolor="#FFFFFF"></body></html>');
            }

            // Azonosító meghatározása a plugin elérési útjából.
            $this->ID = explode(DIRECTORY_SEPARATOR, rtrim(plugin_dir_path(dirname(__file__)), DIRECTORY_SEPARATOR));
            $this->ID = array_pop($this->ID);

        }

        /**
         * JSportnaptar898_cache::getErrors()
         * 
         * A hibák lekérdezése 
         * 
         * @return HTML A hibák szövegesen
         */
        public function getErrors() {
            return implode("\r\n", $this->errors);
        }

        
        /**
         * JSportnaptar898_cache::load()
         * 
         * Adatok beolvasása fájlból
         * 
         * @param string $file A fájl neve, amit a cache-ból be akarunk olvasni
         * @return mixed A fájl tartalma
         */
        public function load($file, $lastmod = null) {
            if (empty($file)) { 
                $this->errors[] = "\r\n" . '<!-- ['.$this->ID.' - cache]:' . __('Filename is empty!', $this->ID) . '-->' . "\r\n";
                return false; 
            }

            if (file_exists($this->path . $file)) {
                $time = filemtime($this->path . $file);
                if (($time > strtotime('-'.$this->timeout.' hours')) or ((!empty($lastmod)) and ($lastmod < $time))) {
                    $content = file_get_contents($this->path . $file);
                    return $content;
                } 
            } 
            
            return false;
        }
        
        /**
         * JSportnaptar898_cache::getArchive()
         * 
         * Adatok beolvasása fájlból akkor is, ha a fájl már lejárt
         * 
         * @param string $file A fájl neve, amit a cache-ból be akarunk olvasni
         * @return mixed A fájl tartalma
         */
        public function getArchive($file) {
            if (empty($file)) { 
                $this->errors[] = "\r\n" . '<!-- ['.$this->ID.' - cache]:' . __('Filename is empty!', $this->ID) . '-->' . "\r\n";
                return false; 
            }

            if (file_exists($this->path . $file)) {
                $content = file_get_contents($this->path . $file);
            } else {
                $this->errors[] = "\r\n" . '<!-- ['.$this->ID.' - cache]:' . __('Archive cache not found!', $this->ID) . '-->' . "\r\n";
            }
            
            return false;
        }
        
        /**
         * JSportnaptar898_cache::store()
         * 
         * Adatok kiírása a gyorsítótárba
         * 
         * @param string $file A fájl neve, ahová ki akarjuk írni a tartalmat
         * @param mixed $data Amit a fájlba akarunk írni
         * @return
         */
        public function store($file, $data) {

            if (empty($file)) { 
                $this->errors[] = "\r\n" . '<!-- ['.$this->ID.' - cache]:' . __('Filename is empty!', $this->ID) . '-->' . "\r\n";
                return false; 
            }
            
            // A teljes útvonalon - ha van - ellenőrizzük, hogy létezik-e a mappa
            $info = pathinfo($file);
            $path = trim($info['dirname'], '. ');
            if (!empty($path)) {
                $path = explode(DIRECTORY_SEPARATOR, $path);
                $tmp = '';
                for ($i = 0; $i <= count($path)-1; $i++) {
                    if (!empty($path[$i])) {
                        $tmp .= $path[$i] .DIRECTORY_SEPARATOR;
                        if (!is_dir($this->path . $tmp)) {
                            // Ha nem létezik, létrehozzuk
                            mkdir($this->path . $tmp, 0775);
                            file_put_contents($this->path . $tmp . 'index.html', '<html><body bgcolor="#FFFFFF"></body></html>');
                        }
                    }
                }
            }
            
            file_put_contents($this->path . $file, $data);
            
        }

    }