<?php

    if ( !defined( 'ABSPATH' ) ) exit;

    /**
     * JSportnaptar898_curl
     * 
     * cURL-en keresztül adatok beolvasása
     * 
     * @package Sportnaptár plugin  
     * @author InfoScope Kft.
     * @copyright Minden jog fenntartva!
     * @version 2020
     * @access public
     */
    class JSportnaptar898_curl extends stdClass {

        public $ID = '';            // Plugin slug a nyelvi fájlokhoz

        private $url = '';          // Lekérdezés linkje
        private $errors = array();  // Hibabejegyzések tömbje
        private $exists = true;     // Elérhető-e a cURL?

        /**
         * JSportnaptar898_curl::__construct()
         * 
         * @param string $url A lekérdezés linkje
         * @return void
         */
        public function __construct($url = '') {
        
            $this->url = $url;    

            // Azonosító meghatározása a plugin elérési útjából.
            $this->ID = explode(DIRECTORY_SEPARATOR, rtrim(plugin_dir_path(dirname(__file__)), DIRECTORY_SEPARATOR));
            $this->ID = array_pop($this->ID);

            if (!in_array('curl', get_loaded_extensions())) {
                $this->errors[] = 
                    "\r\n" . '<!-- ['.$this->ID.' - cURL]:' . "\r\n" .
                           "\t" . __('The cURL plugin is not available!', $this->ID) . "\r\n" .
                           "\t" . __('The plugin uses this plugin for the query, so its existence is absolutely necessary.', $this->ID) . "\r\n" .
                           '-->' . "\r\n";
                $this->exists = false;
            }

        
        }

        /**
         * JSportnaptar898_curl::getErrors()
         * 
         * A hibák lekérdezése 
         * 
         * @return HTML A hibák szövegesen
         */
        public function getErrors() {
            return implode("\r\n", $this->errors);
        }


        /**
         * JSportnaptar898_curl::load()
         * 
         * A szerver lekérdezése és a válasz visszaadása
         * 
         * @return array A szerver válasza
         */
        public function load() {

            if (!$this->exists) { return false; }

            if (empty($this->url)) {
                $this->errors[] = "\r\n" . '<!-- ['.$this->ID.' - cURL]:' . __('URL is empty!', $this->ID) . '-->' . "\r\n";
                return false;
            }

            $cURLConnection = curl_init();
            curl_setopt($cURLConnection, CURLOPT_URL, $this->url);
            curl_setopt($cURLConnection, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($cURLConnection, CURLOPT_CONNECTTIMEOUT, 0); 
            curl_setopt($cURLConnection, CURLOPT_TIMEOUT, 30);            
            
            $response = curl_exec($cURLConnection);

            if ($response !== false) {
                
                $response = str_replace('\n', '<br />', $response);
                $response = json_decode($response, true);
    			
    			if (isset($response['code']) and ($response['code'] == 'rest_no_route')) {
    				// Hiba történt a kommunikáció során
                    $this->errors[] =
                        "\r\n" . '<!-- ['.$this->ID.' - cURL]: ' . __('Failed connection to server', $this->ID) . ' ('.curl_error($cURLConnection).')' . ' -->' . "\r\n";
                    curl_close($cURLConnection);
                    return false;
    			} else {
                    curl_close($cURLConnection);
                    // A kommunikáció sikeres volt, de üres válasz jött vissza a szerverről
                    if (empty($response)) {
                        $this->source = 'error';
                        $this->errors[] =
                            "\r\n" . '<!-- ['.$this->ID.' - cURL]:' . "\r\n" .
                            "\t" . __('Empty answer!', $this->ID) . "\r\n" .
                            "\t" . __('Communication was successful, but the server returned an empty response. Are you sure the link is good?', $this->ID) . "\r\n" .
                            '-->' . "\r\n";
                        return false;
                    } else {
                        return $response;
                    }
                }
            } 
            $this->errors[] =
                "\r\n" . '<!-- ['.$this->ID.' - cURL]: ' . __('Failed connection to server', $this->ID) . ' ('.curl_error($cURLConnection).')' . ' -->' . "\r\n";
            curl_close($cURLConnection);
            return false;
        }
        

    }