<?php

    if ( !defined( 'ABSPATH' ) ) exit;

    if (!class_exists('JSportnaptar898_cache')) {
        include_once( dirname(__file__) .DIRECTORY_SEPARATOR. 'cache.php');
    }

    if (!class_exists('JSportnaptar898_one')) {
        include_once( dirname(__file__) .DIRECTORY_SEPARATOR. 'one.php');
    }

class JSportnaptar898_list extends stdClass{
    
    private $options = null;
    private $call_params = array('category', 'hashtag', 'owner');
    private $errors  = array();
    private $content = '';
    private $cache   = null;
    private $events  = array();

    public function __construct($options) {

        $is_valid = false;
        $url = array();
        foreach ($options as $key => $value) {
            if (in_array($key, $this->call_params)) {
                $url[] = $key . '='.$value.'';
                $is_valid = true;
            }
        }
        // $this->call = '<code>[sportnaptar ' . implode(' ', $calls) . ']</code><br />';
        if (!$is_valid) {
            $this->errors[] = "\r\n" . '<!-- [sportnaptar]: ' . __('Not valid query parameters!', 'sportnaptar') . ' -->' . "\r\n";
        } else {

            // Paraméterek beállítása            
            $this->options = $options;
            $this->options['token'] = $this->getToken();

            // Ha nincs beállított sablon, akkor az alapértelmezett kerül beállításra
            if (!isset($this->options['template'])) {
                $this->options['template'] = 'default/list';
            } else {
                $this->options['template'] = $this->options['template'].'/list';
            }

            // Ha nincs beállított nyelv, akkor a magyar kerül beállításra
            if (!isset($this->options['lang'])) {
                $this->options['lang'] = 'hu';
            }
            

            $filename = array();
            if (isset($options['category']) and (!empty($options['category']))) {
                $filename[] = $options['category'];
            }
            if (isset($options['owner']) and (!empty($options['owner']))) {
                $filename[] = $options['owner'];
            }
            if (isset($options['hashtag']) and (!empty($options['hashtag']))) {
                $filename[] = $this->getAlias($options['hashtag']);
            }
            $this->options['id'] = implode('_', $filename);
            
            // Első lépésben megnézzük, hogy a lista a gyorsítótárban rendelkezésre áll-e?
            $this->cache = new JSportnaptar898_cache($this->options['cache_time']);

            $json = $this->cache->load($this->options['id'].'.json'); 
            if (empty($json)) {
                // Nem áll rendelkezésre, beolvassuk
                $json = $this->getJSONData($url);
                if (!empty($json)) {
                    // A frissen lekérdezett adatokat kiírjuk a gyorsítótárba.
                    $this->cache->store($this->options['id'].'.json', $json); 
                } else {
                    // Beolvasás lejárt gyorsítótárból
                    $json = $this->cache->getArchive($this->options['id'].'.json'); 
                    if ($json === false) {
                        $this->errors[] = $this->cache->getErrors();
                        $json = '';
                    }
                }
            } else {
                $this->source = 'json';
            }

            if (empty($json)) {
                exit;
            }

            // Meg kell hívni az alapértelmezett stíluslapokat is, hogy a css és js fájlok betöltődjenek.
            // Korábban, mint a lista saját stílusát.
            $options = array(
                    'template' => str_replace('list', 'one', $this->options['template'])
                );
            $tmpl = new JTemplates($options);
            $tmpl->getCSSandJS($options['template']);
            

            $json = unserialize($json);

            // Aktuális beállítások alapján sorba rendezzük
            if ($this->options['sort'] == 'date') {
                usort($json, function($a, $b) { 
                        return strcmp($a['start_date'], $b['start_date']); 
                    });
            } else {
                usort($json, function($a, $b) { 
                        return strcmp($b['modified'], $a['modified']); 
                    });
            }

            for ($i = 0; $i <= count($json)-1; $i++) {
               
                if ($this->options['hide_behind'] != 1) {
                    $this->events[] = $json[$i];
                } else {
                    if ($json[$i]['postponed'] == 0) {
                        $this->events[] = $json[$i];
                    }
                }
            } 

            for ($i = 0; $i <= count($this->events)-1; $i++) {
                // Render carts                
                $this->options['id'] = $this->events[$i]['uid'];
                $item = new JSportnaptar898_one($this->options, $this->events[$i]['modified'], true);
                $this->content .= $item->getContentHTML();
            }
        }
    }

    private function getJSONData($params) {
        
        $query_url = SPORTNAPTAR_898_REMOTE_LIST_URL . '?' . implode('&',$params);

        // Adatok cURL lekérdezése
        if (!class_exists('JSportnaptar898_curl')) {
            include_once( dirname(__file__) .DIRECTORY_SEPARATOR. 'curl.php');
        }
        $curl = new JSportnaptar898_curl($query_url);

        $events = $curl->load();

        if ($events === false) {
            // Hiba beírva a hibák tömbjébe
            $this->errors[] = $curl->getErrors();
            // Beolvasás lejárt gyorsítótárból
            $events = $this->cache->getArchive($this->cache_filename); 
            if ($events === false) {
                $this->errors[] = $this->cache->getErrors();
                return array();
            }
        } else {
            $events = serialize($events);
        }
                
        return $events;
    }
    private function getToken() {
        // Egyedi azonosító generálása
        $chars = 'ABCDEFGHIJKLMNOPQURSTXYVWZ0123456789';
        $token = '';
        for ($i = 1; $i <= 10; $i++) {
        	$token .= substr($chars, rand(0, strlen($chars)-1), 1);
        }                
        return $token;
    }

    function getAlias($text) {
        // SEF hashtag a cache fájl névhez
        $str = trim(mb_strtolower($text, 'UTF-8'));

        $accented = array( "á", "é", "í", "ó", "ö", "ő", "ú", "ü", "ű" );  
        $mask     = array( "a", "e", "i", "o", "o", "o", "u", "u", "u" );  
        $str = str_replace( $accented, $mask, $str );  

        $str = str_replace('-', ' ', $str);
        $str = str_replace('.', ' ', $str);
        $str = str_replace('_', ' ', $str);
        $str = preg_replace(array('/\s+/','/[^A-Za-z0-9\-]/'), array('-',''), $str);
        
        return $str;
    }


    function getContent() {

        $html = "\r\n" .
                // $this->call . 
                '<div id="sportnaptar_'.$this->options['token'].'" 
                      class="sportnaptar-list-block" 
                      data-plugin="sportnaptar"
                      data-template="'.$this->options['template'].'"  
                      data-token="'.$this->options['token'].'">' .
                    $this->content .  "\r\n" .
                '</div>' . "\r\n";
        if (WP_DEBUG) {
            $html .= implode('', $this->errors);
        } 
        $html .= '<div style="clear:both"></div>';
        
        return $html;
    }

}
    