<?php

    if ( !defined( 'ABSPATH' ) ) exit;

    if (!class_exists('JSportnaptar898_cache')) {
        include_once( dirname(__file__) .DIRECTORY_SEPARATOR. 'cache.php');
    }


class JSportnaptar898_one extends stdClass{

    private $content = '';
    private $options = array();
    private $lastmod = null;
    private $cache   = null;

    private $errors = array();

    /**
     * JSportnaptar898_one::__construct()
     * 
     * @param mixed $options A meghíváskor átadott paraméterek tömbje
     * @param mixed $lastmod Az utolsó módosítás időbélyege
     * @return void
     */
    public function __construct($options, $lastmod = null, $full_template = false) {

        if (!empty($lastmod)) { $this->lastmod = $lastmod; }

        $this->options = $options;
        $this->options['token'] = $this->getToken();

        // Ha nincs beállított sablon, akkor az alapértelmezett kerül beállításra
        if (!isset($this->options['template'])) {
            $this->options['template'] = 'default/one';
        } else {
            if (!$full_template) {
                $this->options['template'] = $this->options['template'].'/one';
            }
        }

        // Ha nincs beállított nyelv, akkor a magyar kerül beállításra
        if (!isset($this->options['lang'])) {
            $this->options['lang'] = 'hu';
        }

        // Első lépésben megnézzük, hogy a generált HTML a gyorsítótárban rendelkezésre áll-e?
        $this->cache = new JSportnaptar898_cache($this->options['cache_time'], $this->lastmod);
        $this->content = $this->cache->load($this->options['template'] .DIRECTORY_SEPARATOR. $this->options['id'].'.html'); 

        if (empty($this->content)) {
            // Nem áll rendelkezésre, vagy már lejárt.
            
            // Megnézzük, hogy az adatok JSON fájlként rendelkezésre állnak-e a gyorsítótárban
            // Előfordulhat, hogy egy másik sablon által korábban le lett töltve
            
            $json = $this->cache->load($this->options['id'].'.json'); 
            if (empty($json)) {
                // Nem áll rendelkezésre, beolvassuk
                $json = $this->getJSONData();
                if (!empty($json)) {
                    // A frissen lekérdezett adatokat kiírjuk a gyorsítótárba.
                    $this->cache->store($this->options['id'].'.json', $json); 
                } else {
                    // Beolvasás lejárt gyorsítótárból
                    $json = $this->cache->getArchive($this->options['id'].'.json'); 
                    if ($json === false) {
                        $this->errors[] = $this->cache->getErrors();
                        $json = '';
                    }
                }
            } else {
                $this->source = 'json';
            }

            // Ha rendelkezésre állnak az adatok, akkor legyártjuk a HTML állományt.
            if (!empty($json)) {
                $this->content = $this->prepareContent($json);
                // Kiírjuk a gyorsítótárba
                $this->cache->store($this->options['template'] .DIRECTORY_SEPARATOR. $this->options['id'].'.html', $this->content);                
            }
        } else {
            // Ha a HTML rendelkezésre áll, akkor gondoskodni kell a stíluslapok betöltéséről.
            $tmpl = new JTemplates($this->options);
            $tmpl->getCSSandJS($this->options['template']);
            $this->source = 'cache';
        }
    }


    // A sablon beolvasása és az adatok behelyettesítése
    private function prepareContent($data) {

        $data = unserialize($data);

        if (substr($this->options['template'], -5) != '.html' ) {
            $data['template'] = $this->options['template'] . '/template.html';
        } else {
            $data['template'] = $this->options['template'];
        }
        
        $template = new My_Templates($data);                
        $template->categories = $this->getCategories();;
        
        $content = $template->prepareContent();
        
        return $content;           
    }

    private function getCategories() {
    // Kategóriák betöltése gyorsítótárból, szükség szerint frissítése
        
        $filename = $this->options['lang'].'.categories.json';
        
        $data = $this->cache->load($filename);
        if (!empty($data)) {
            $this->categories = unserialize($data);
            return $this->categories;    
        }

        if (!class_exists('JSportnaptar898_curl')) {
            include_once( dirname(__file__) .DIRECTORY_SEPARATOR. 'curl.php');
        }
        $url = SPORTNAPTAR_898_REMOTE_CATEGORY_URL . '?language='.$this->options['lang'];
        $curl = new JSportnaptar898_curl($url);

        $response = $curl->load();
        if ($response === false) {
            // Hiba beírva a hibák tömbjébe
            $this->errors[] = $curl->getErrors();
            // Beolvasás lejárt gyorsítótárból
            $this->categories = $this->cache->getArchive($filename); 
            if ($this->categories === false) {
                $this->errors[] = $this->cache->getErrors();
            }
        } else {
            $this->categories = $response;
            $this->cache->store($filename, serialize($this->categories));    
        }
        return $response;
    }

    private function getJSONData() {

        // Adatok cURL lekérdezése
        if (!class_exists('JSportnaptar898_curl')) {
            include_once( dirname(__file__) .DIRECTORY_SEPARATOR. 'curl.php');
        }
        $url = SPORTNAPTAR_898_REMOTE_URL . '?id='.$this->options['id'].'&amp;language='.$this->options['lang'];
        $curl = new JSportnaptar898_curl($url);

        $response = $curl->load();
        if ($response === false) {
            // Hiba beírva a hibák tömbjébe
            $this->errors[] = $curl->getErrors();
            $response = '';
        } else {
            $response['id'] = $this->options['id'];
            $response = serialize($response);
        }
        return $response;        
    }

    private function getToken() {
        // Egyedi azonosító generálása
        $chars = 'ABCDEFGHIJKLMNOPQURSTXYVWZ0123456789';
        $token = '';
        for ($i = 1; $i <= 10; $i++) {
        	$token .= substr($chars, rand(0, strlen($chars)-1), 1);
        }                
        return $token;
    }

    function getContentHTML() {
        return $this->content;
    }

    function getContent() {

            $html = "\r\n" .
                    '<div id="sportnaptar_'.$this->options['token'].'" 
                          class="sportnaptar" 
                          data-plugin="sportnaptar" 
                          data-token="'.$this->options['token'].'" 
                          data-id="'.$this->options['id'].'">' . "\r\n" .
                        $this->content .  "\r\n" .
                    '</div>' . "\r\n" . 
                    implode("\r\n", $this->errors) . "\r\n" .
                    '<div style="clear:both"></div>';
            
            return $html;
    }
}
