<?php

if ( !defined( 'ABSPATH' ) ) exit;

function sportnaptar_898_get_cache_files($path = '') {
    
    $tilt    = array('index.html', '.', '..');
    $count   = 0;

    if (empty($path)) {
        $path = SPORTNAPTAR_898_ROOT .DIRECTORY_SEPARATOR. 'cache';
    }

    // A cache mappa alatti mappák beolvasása
    $handle = @opendir($path);
    if ($handle) {
        while (false !== ($item = readdir($handle))) {

            if (!in_array($item, $tilt)) {
                
                if (is_dir($path .DIRECTORY_SEPARATOR. $item)) {
                    $count += sportnaptar_898_get_cache_files($path . '/' . $item);
                } else {
                    $count++;
                }
            
            }
            
        }        
    }

    return $count;    
}

class sportnaptar_898_settings {
    
    private $options;

    public function __construct()
    {
        add_action( 'admin_menu', array( $this, 'add_plugin_page' ) );
        add_action( 'admin_init', array( $this, 'page_init' ) );
    }

    public function add_plugin_page()
    {
        // This page will be under "Settings"
        add_options_page(
            'Settings Admin', 
            __('Sportnaptár settings', 'sportnaptar'), 
            'manage_options', 
            'sportnaptar-setting-admin', 
            array( $this, 'create_admin_page' )
        );
    }

    public function create_admin_page()
    {
        $this->options = sportnaptar_898_get_options ( 'sportnaptar_options' );
        ?>
        <style>.form-table td {padding: 0px 10px 0 10px;}</style>
        <div class="wrap">
            <h1><?php echo __('Sportnaptar settings', 'sportnaptar');?></h1>
            <div class="sportnaptar-setting-page">
                <div>
                    <form method="post" action="options.php">
                    <?php
                        // This prints out all hidden setting fields
                        settings_fields( 'sportnaptar_option_main' );
                        do_settings_sections( 'sportnaptar-setting-admin' );
                        submit_button();
                        
                        $cache_files_count = sportnaptar_898_get_cache_files();
                        /* if ($cache_files_count > 0) :  */ ?> 
                            <div id="delete_cache_block">
                                <hr />
                                <p><?php echo sprintf( __('%s file(s) in cache', 'sportnaptar'), $cache_files_count);?></p>
                                <button onclick="sportnaptar_898_delete_cache();" type="button" id="delete_cache_button" class="button button-secondary">
                                    <?php echo __('Delete cache', 'sportnaptar');?>
                                </button>
                            </div>
                        <?php  /* endif; */ ?>   
                    </form>
                </div>
            </div>
        </div>
        <?php
    }

    /**
     * Register and add settings
     */
    public function page_init()
    {        
        register_setting(
            'sportnaptar_option_main', // Option group
            'sportnaptar_options', // Option name
            array( $this, 'sanitize' ) // Sanitize
        );

        add_settings_section(
            'sportnaptar_settings_main', // ID
            __('Insert the content', 'sportnaptar'), // Title
            array( $this, 'print_section_info' ), // Callback
            'sportnaptar-setting-admin' // Page
        );  

        add_settings_field(
            'my_page', // ID
            __('My profile page', 'sportnaptar'), // Title
            array( $this, 'my_page_callback' ), // Callback
            'sportnaptar-setting-admin', // Page
            'sportnaptar_settings_main' // Section
        );
        add_settings_field(
            'cache_time', // ID
            __('Cache time', 'sportnaptar'), // Title
            array( $this, 'cache_time_callback' ), // Callback
            'sportnaptar-setting-admin', // Page
            'sportnaptar_settings_main' // Section
        );

        add_settings_field(
            'hide_behind', // ID
            __('Hide behind', 'sportnaptar'), // Title
            array( $this, 'hide_behind_callback' ), // Callback
            'sportnaptar-setting-admin', // Page
            'sportnaptar_settings_main' // Section
        );

        add_settings_field(
            'sort', // ID
            __('Sort', 'sportnaptar'), // Title
            array( $this, 'sort_callback' ), // Callback
            'sportnaptar-setting-admin', // Page
            'sportnaptar_settings_main' // Section
        );

        add_settings_field(
            'template', // ID
            __('Template', 'sportnaptar'), // Title
            array( $this, 'template_callback' ), // Callback
            'sportnaptar-setting-admin', // Page
            'sportnaptar_settings_main' // Section
        );

    }

    public function sanitize( $input )
    {
        $new_input = array();

        if ( isset( $input['my_page'] ) ) {
            $new_input['my_page'] = trim( $input['my_page'] );
        }

        if ( isset( $input['cache_time'] ) ) {
            $new_input['cache_time'] = trim( $input['cache_time'] );
        }

        if ( isset( $input['hide_behind'] ) ) {
            $new_input['hide_behind'] = 1;
        } else {
            $new_input['hide_behind'] = 0;
        }

        if ( isset( $input['sort'] ) ) {
            $new_input['sort'] = trim( $input['sort'] );
        }

        if ( isset( $input['template'] ) ) {
            $new_input['template'] = trim( $input['template'] );
        }

        return $new_input;
    }

    public function get_templates() {

        if (!class_exists('JDovi42Files')) {
            include_once(dirname(__file__) .DIRECTORY_SEPARATOR. 'jdovi42_files.php');
        }

        $source = new JDovi42Files(get_stylesheet_directory() . '/sportnaptar');
        $tmp = $source->getFolders();        
        if (!empty($tmp)) {
            for ($i = 0; $i <= count($tmp)-1; $i++) {
            	$folders[$tmp[$i]['filename']] = 'Theme: ' . __($tmp[$i]['title'], 'sportnaptar'); 
            }
        }

        $source = new JDovi42Files(SPORTNAPTAR_898_ROOT . '/templates'); 
        $tmp = $source->getFolders();        
        if (!empty($tmp)) {
            for ($i = 0; $i <= count($tmp)-1; $i++) {
            	if (!isset($folders[$tmp[$i]['filename']])) {
                    $folders[$tmp[$i]['filename']] = __($tmp[$i]['title'], 'sportnaptar'); 
            	}
            }
        }
        ksort($folders);

        $html = '<select name="sportnaptar_template" class="sportnaptar_template">';
        foreach ($folders as $key => $value) {
            $html .= '<option value="'.$key.'"';
            if ($this->options['template'] == $key) {
               $html .= ' SELECTED';
            }
            $html .= '>'.$value.'</option>';
        }
        $html .= '</select>';
        return $html;
    }
    
    public function print_section_info() {
                            
        $lang_select = '<select class="sportnaptar_lang" name="sportnaptar_lang">' . 
                       '<option value="hu">'.__('hu', 'sportnaptar').'</option>'. 
                       '<option value="en">'.__('en', 'sportnaptar').'</option>'.
                       '</select>'; 
        

        //Kategóriák
        
        $lang = get_bloginfo('language');
        if ($lang != 'hu') { $lang = 'en'; }
        
        if (!class_exists('JSportnaptar898_cache')) {
            include_once( dirname(__file__) .DIRECTORY_SEPARATOR. 'cache.php');
        }
        $categories = array();
        $filename = $lang.'.categories.json';

        $cache = new JSportnaptar898_cache($this->options['cache_time']);        
        
        $data = $cache->load($filename);
        if (!empty($data)) {
            $categories = unserialize($data);
        } else {
            if (!class_exists('JSportnaptar898_curl')) {
                include_once( dirname(__file__) .DIRECTORY_SEPARATOR. 'curl.php');
            }
            $url = SPORTNAPTAR_898_REMOTE_CATEGORY_URL . '?language='.$lang;
            $curl = new JSportnaptar898_curl($url);

            $response = $curl->load();
            if ($response === false) {
                $categories = $cache->getArchive($filename); 
            } else {
                $categories = $response;
                $cache->store($filename, serialize($categories));    
            }
        }

        $category_select = '<select class="sportnaptar_category" name="sportnaptar_lang">';
        $category_select .= '<option value="">--== '.__('Category not set', 'sportnaptar').' ==--</option>'; 

        usort($categories, function($a, $b) { 
                return strcmp(strtolower($a['name']), strtolower($b['name'])); 
            });
        
        if (!empty($categories)) {
            foreach ($categories as $category) {
                $category_select .= '<option value="'.$category['id'].'">'.$category['name'].'</option>'; 
            }
        }

        $category_select .= '</select>';

        ?>
        <div class="wrap">
            <h2 class="nav-tab-wrapper">
                <a href="#" onclick="sportnaptar_898_showtab('sportnaptar_one_event');" class="sportnaptar_one_event nav-tab  nav-tab-active"><?php echo __('One event', 'sportnaptar');?></a>
                <a href="#" onclick="sportnaptar_898_showtab('sportnaptar_event_list');"class="sportnaptar_event_list nav-tab"><?php echo __('Event list', 'sportnaptar');?></a>
            </h2>
            <div id="sportnaptar_shorcodes" style="padding: 20px 0; border-bottom:2px solid #ccc;">

                <div id="sportnaptar_one_event">
                    
                    <div class="parameter">
                    <?php echo __('Template:', 'sportnaptar');?>:
                    <?php 
                        $template_list = $this->get_templates(); 
                        echo $template_list;                    
                    ?>
                    </div>

                    <div class="parameter">
                        <?php echo __('URL:', 'sportnaptar');?>
                        <input type="text" class="event_url" name="event_url" value="" style="width:300px;" />
                    </div>
                    <div class="parameter">
                        <?php echo __('Language:', 'sportnaptar');?>
                        <?php echo $lang_select;?>
                    </div>
                    <div style="clear:both;"></div>
                    <div style="margin-top: 20px;">
                        Shortcode: <span class="shortcode"></span>
                        <button onclick="sportnaptar_898_to_clipboard_one_event()" type="button" class="button button-primary">
                        Másol
                        </button>
                    </div>
                    <div class="shortcode-one-error"><?php echo __('Shortcode not valid!', 'sportnaptar');?></div>
                </div>

                <div id="sportnaptar_event_list" style="display: none;">

                    <div class="parameter">
                    <?php echo __('Template:', 'sportnaptar');?>:
                    <?php 
                        echo $template_list;                    
                    ?>
                    </div>

                    <div class="parameter">
                        <?php echo __('Category:', 'sportnaptar');?>
                        <?php echo $category_select;?>
                    </div>
                    <div class="parameter">
                        <?php echo __('Hashtag:', 'sportnaptar');?>
                        <input type="text" class="event_hashtag" name="event_hashtag" value="" style="width:100px;" />
                    </div>
                    <?php
                        // Utolsó blokk kikeresése
                        $temp = explode(DIRECTORY_SEPARATOR, $this->options['my_page']);
                        $temp = $temp[count($temp)-1];
                        
                        // Az utolsó blokkból az azonosító kiemelése
                        $temp = explode('_', $temp);
                        $temp = $temp[count($temp)-1];
                
                        $myID = $temp;
                    ?>
                    <?php if (!empty($myID)) : ?>
                    <div class="parameter">
                        <input type="checkbox" class="my_event" name="my_event" value="<?php echo $myID;?>" />
                        <label for="my_event"><?php echo __('My own events', 'sportnaptar');?></label>
                    </div>
                    <?php endif; ?>

                    <div class="parameter">
                        <?php echo __('Language:', 'sportnaptar');?>
                        <?php echo $lang_select;?>
                    </div>
                    <div style="clear:both;"></div>
                    <div style="margin-top: 20px;">
                        Shortcode: <span class="shortcode"></span>
                        <button onclick="sportnaptar_898_to_clipboard_event_list()" type="button" class="button button-primary">
                        Másol
                        </button>
                    </div>
                    <div class="shortcode-list-error"><?php echo __('Shortcode not valid!', 'sportnaptar');?></div>
                </div>
            </div>        
        </div>
        <?php
        echo '<h2>'.
             __('Plugin setting', 'sportnaptar') .
             '</h2>';
    }

    public function my_page_callback()
    {
        printf(
            '<input type="text" name="sportnaptar_options[my_page]" value="%s" style="width:300px;" />' .
            '<br /><small>'.__('My profile page within sportnaptar.hu', 'sportnaptar').'</small>',
            isset( $this->options['my_page'] ) ? esc_attr( $this->options['my_page']) : ''
        );
    }


    public function cache_time_callback()
    {
        printf(
            '<input type="number" name="sportnaptar_options[cache_time]" value="%s"  min="1" max="99" />' .
            '<br /><small>'.__('Cache life in hours.', 'sportnaptar').'</small>',
            isset( $this->options['cache_time'] ) ? esc_attr( $this->options['cache_time']) : ''
        );
    }

    public function hide_behind_callback()
    {
        if ($this->options['hide_behind'] == 1) {
            $checked = ' CHECKED ';
        } else {
            $checked = '';
        } 
        printf(
            '<input type="checkbox" id="teszt_hide_behind" name="sportnaptar_options[hide_behind]" value="1" '.$checked.' />' .
            '<label for="sportnaptar_hide_behind"></label>' .
            '<small>'.__('Hide behind event from cards.', 'sportnaptar').'</small>',
            isset( $this->options['hide_behind'] ) ? esc_attr( $this->options['hide_behind']) : ''
        );
    }

    public function sort_callback()
    {
        echo '<!-- ' . __('date_title', 'sportnaptar') . ' -->';
        echo '<!-- ' . __('new_title', 'sportnaptar') . ' -->';
        $values = array('date', 'new');
        $html = '<select name="sportnaptar_options[sort]">';
        foreach ($values as $item) {
            $html .= '<option value="'.$item.'"';
            if ($this->options['sort'] == $item) {
               $html .= ' SELECTED';
            }
            $html .= '>'.__($item.'_title', 'sportnaptar').'</option>';
        }
        $html .= '</select>';
        $html .= '<br /><small>'.__('Sort list order', 'sportnaptar').'</small>';
        echo $html;
    }

    public function template_callback()
    {
        $html = $this->get_templates();
        $html = str_replace('name="sportnaptar_template"', 'name="sportnaptar_options[template]"', $html);
        $html .= '<br /><small>'.__('Default template', 'sportnaptar').'</small>';
        echo $html;
    }


}
