<?php

    if ( !defined( 'ABSPATH' ) ) exit;

    if (!class_exists('JSportnaptar898_templates')) {
        include_once( dirname(__file__) .DIRECTORY_SEPARATOR. 'templates.php');
    }

    class My_Templates extends JTemplates {

        public $date_format = '';
        public $time_format = '';
        public $unit_type = 0;
        public $categories = array();

        public function __construct($options = array()) {
            
            $this->date_format = get_option( 'date_format' );
            $this->time_format = get_option( 'time_format' );

            // Az aktivitások adatai manipulálása
            if (isset($options['activities'])) {
                for ($i = 0; $i <= count($options['activities'])-1; $i++) {
                    $options['activities'][$i] = $this->modifyInitData($options['activities'][$i]);                	
                }
            }
            
            parent::__construct($options);
        }

    	function getShortDescription($html, $size = 100) {
            
            $html = strip_tags($html);
            $html = strip_shortcodes($html);
            $source = explode(' ', $html);
            $tmp = '';
            $i = 0;
            $ok = true;
            while ((isset($source[$i])) and (mb_strlen($tmp . ' ' . $source[$i], 'UTF-8') < $size) and ($i <= (count($source)-1))) {
                $tmp .= ' ' . $source[$i];
                $i++;
            }
            if (mb_strlen($html) > $size) {
                $tmp .= ' [...]';
            }
            return $tmp;
    	}


        function preProcess($data) {

            if (isset($data['description']) and (!empty($data['description']))) {
                $text = trim(strip_tags($data['description']));
                $text = str_replace('#', '', $text);
                
                $data['short_description'] = $this->getShortDescription($text);
            }

            // hashtags alapján mutatott módosítás a description változóban
            if (isset($data['hashtags']) and (!empty($data['hashtags']))) {

                $hashtags = array('template' => $data['template_folder'] . 'hashtags.html');
                $tmpl = new JTemplates($hashtags);

                $hash_template = $tmpl->getTemplate('hashtags.html');
                $hashs = array();
                $hashtags = explode(',', $data['hashtags']);
                $data['hashtags'] = $hashtags;
                foreach ($hashtags as $hash) {
                    $tmpl = $hash_template;
                    $tmpl = $this->conditionalTemplate($tmpl, array('hashtag' => $hash));
                    $tmpl = str_replace('<!-- hashtag -->', $hash, $tmpl);
                    $data['description'] = str_replace('#'.$hash, $tmpl, $data['description']);      
                }
                
            }

            // Első és utolsó aktivitás keresése, ha van a tömbben aktivitás tömb.
            if (isset($data['activities'])) {
            
                $distances = array(); 
                for ($i = 0; $i <= count($data['activities'])-1; $i++) {
                    if (isset($data['activities'][$i]['distance'])) {
                        $distances[] = $this->modifyValue('distance', $data['activities'][$i]['distance']);
                    }
                }
                $data['distances'] = implode(' / ', $distances);
                
                if (isset($data['activities'][0]['start_address'])) {
                    $data['location'] = $data['activities'][0]['start_address'];
                }                
                // Az első aktivitás bejegyzés lekérdezése
                $first = '999999999999999999999999999999999999999';
                // Nem biztos, hogy időrendi sorrendben vannak
                for ($i = 0; $i <= count($data['activities'])-1; $i++) {
                    if ($data['activities'][$i]['startdate'] < $first) {
                        $data['firsttime'] = $data['activities'][$i]['startdate'];
                    }
                }
    
    
                // Az utolsó aktivitás bejegyzés lekérdezése
                $last = null;
                // Nem biztos, hogy időrendi sorrendben vannak
                for ($i = 0; $i <= count($data['activities'])-1; $i++) {
                    if ($data['activities'][$i]['enddate'] > $last) {
                        $data['lasttime'] = $data['activities'][$i]['enddate'];
                        $last = $data['activities'][$i]['enddate'];
                    }
                }
    
                $data['outtime'] = '';
                // Ha az utolsó zárás időpontja elmúlt, akkor a jelző értékét "nem üres"-re állítjuk
                if (strtotime($last) < time()) {
                    $data['outtime'] = '1';
                }
            }

            if (isset($data['unit_type']) and (!empty($data['unit_type']))) {
                $this->unit_type = $data['unit_type'];
            }

            // Kategóriára vonatkozó bejegyzések 
            if (isset($data['category_id'])) {
                $data['category_icon'] = $this->categories[$data['category_id']]['icon'];
                $data['category_name'] = $this->categories[$data['category_id']]['name'];
                if (isset($data['type']) and (!empty($data['type']))) {
                    $data['category_type'] = $this->categories[ $data['category_id']]['types'][$data['type']];
                } else {
                    $data['category_type'] = '';
                }
            }

            // Ha van "links" és "descriptions" blokk, akkor a linkeket áthelyezzük oda
            if (isset($data['links']) and (!empty($data['links'])) and isset($data['descriptions'])) {
                $data['descriptions'][] = array(
                    'name' => 'Linkek',
                    'links' => $data['links']
                );
                unset($data['links']);
            }

            // Ha van "documents" és "descriptions" blokk, akkor a dokumentumokat áthelyezzük oda
            if (isset($data['documents']) and (!empty($data['documents'])) and isset($data['descriptions'])) {
                $data['descriptions'][] = array(
                    'name' => 'Dokumentumok',
                    'documents' => $data['documents']
                );
                unset($data['documents']);
            }

            if (isset($data['descriptions']) and (!empty($data['descriptions']))) {
                $data['descriptions_tabs'] = $data['descriptions'];                   
            }
            
            return $data;
        }


        function modifyInitData($data) {
            
            // Csak akkor, ha az adatok tömb.
            if (is_array($data)) {
                
                // Ha csak "Start" van, de nincs cél, akkor ez HELYETTE helyszín "locate" lesz
                if ((isset($data['start_address']) and (!empty($data['start_address']))) and
                    (isset($data['end_address']) and (empty($data['end_address'])))) {
                        $data['location_address'] = $data['start_address'];
                        $data['start_address'] = '';
                        // Ha a koordináták adottak, akkor generálunk belőle egy linket.
                        if (isset($data['start_address_lat']) and (!empty($data['start_address_lat'])) and 
                            isset($data['start_address_lng']) and (!empty($data['start_address_lng']))) {
                                $data['location_map_link'] = 'https://www.google.hu/maps/place/'.$data['location_address'].'/@'.$data['start_address_lat'].','.$data['start_address_lng'].',16z';
                            } else {
                                // Egyébként üres mezőnek kerül beállításra - így a sablonból törlődni fog.
                                $data['location_map_link'] = '';
                            }
                    } else {
                        // Létrehozzuk az elemet üres értékkel, hogy a sablonból törölni lehessen a vonatkozó bejegyzést
                        $data['location_address'] = '';
                    }

                // A starthoz kapcsolódó link létrehozása
                if (isset($data['start_address']) and (!empty($data['start_address']))) {
                    // Ha a koordináták adottak, akkor generálunk belőle egy linket.
                    if (isset($data['start_address_lat']) and (!empty($data['start_address_lat'])) and 
                        isset($data['start_address_lng']) and (!empty($data['start_address_lng']))) {
                            $data['start_map_link'] = 'https://www.google.hu/maps/place/'.$data['start_address'].'/@'.$data['start_address_lat'].','.$data['start_address_lng'].',16z';
                        } else {
                            // Egyébként üres mezőnek kerül beállításra - így a sablonból törlődni fog.
                            $data['start_map_link'] = '';
                        }
                } else {
                    $data['start_map_link'] = '';
                }

                // A célhoz kapcsolódó link létrehozása
                if (isset($data['end_address']) and (!empty($data['end_address']))) {
                    // Ha a koordináták adottak, akkor generálunk belőle egy linket.
                    if (isset($data['end_address_lat']) and (!empty($data['end_address_lat'])) and 
                        isset($data['end_address_lng']) and (!empty($data['end_address_lng']))) {
                            $data['end_map_link'] = 'https://www.google.hu/maps/place/'.$data['end_address'].'/@'.$data['end_address_lat'].','.$data['end_address_lng'].',16z';
                        } else {
                            // Egyébként üres mezőnek kerül beállításra - így a sablonból törlődni fog.
                            $data['end_map_link'] = '';
                        }
                } else {
                    $data['end_map_link'] = '';
                }

                // Ha a távolság nulla, akkor legyen üres érték
                if (isset($data['distance']) and ($data['distance'] == 0)) {
                    $data['distance'] = '';
                }

                // Ha a szint nulla, akkor legyen üres érték
                if (isset($data['level']) and ($data['level'] == 0)) {
                    $data['level'] = '';
                }
                
                // Ha van "links" és "descriptions" blokk, akkor a linkeket áthelyezzük oda
                if (isset($data['links']) and (!empty($data['links'])) and isset($data['descriptions'])) {
                    $data['descriptions'][] = array(
                        'name' => 'Linkek',
                        'links' => $data['links']
                    );
                    unset($data['links']);
                }

                // Ha van "documents" és "descriptions" blokk, akkor a dokumentumokat áthelyezzük oda
                if (isset($data['documents']) and (!empty($data['documents'])) and isset($data['descriptions'])) {
                    $data['descriptions'][] = array(
                        'name' => 'Dokumentumok',
                        'documents' => $data['documents']
                    );
                    unset($data['documents']);
                }
                
            }

            return $data;
        }

        function modifyValue($key, $value) {
            if (!empty($key)) {
                switch ($key) { 
                	case 'startdate':
                	case 'enddate':
                	case 'entrystart':
                	case 'entrydeadline':
                	case 'firsttime':
                	case 'lasttime':
                	   // Szövegesen kapott időpontok
                       $format = $this->date_format;
                       if (!empty($this->time_format)) {
                            $format .= ' ' . $this->time_format;
                       }
                       $value = date_i18n($format, strtotime($value));
                       break;
                	case 'distance':
                	   $t = explode('.', $value);
                       if (isset($t[1])) {
                            $t[1] = rtrim($t[1], '0');
                            if (!empty($t[1])) {
                                $value = $t[0].'.'.$t[1];
                            } else {
                                $value = $t[0];
                            }
                       }
                	   $unit = array('km', 'mile');
                       $value .= $unit[$this->unit_type];
                       break;
                	case 'level':
                	   $t = explode('.', $value);
                       if (isset($t[1])) {
                            $t[1] = rtrim($t[1], '0');
                            if (!empty($t[1])) {
                                $value = $t[0].'.'.$t[1];
                            } else {
                                $value = $t[0];
                            }
                       }
                       $unit = array('m', 'foot');
                       $value .= $unit[$this->unit_type];
                       break;
                	case 'level_time':
                       $value = date_i18n($this->time_format, $value);
                       break;
                    case 'price':
                	   // Számformátum a magyarnak megfelelő
                       $value = number_format($value, 0, '', ' '); 
                       break;
                
                	default :
                }                
            }
            return $value;
        }

        
    }
