<?php

    if ( !defined( 'ABSPATH' ) ) exit;

    /**
     * JTemplates
     * 
     * HTML sablon használata adattartalom megjelenítéséhez WordPress rendszerben
     * A sablonok a plugin mappájában egy "templates" mappában, sablononként külön mappában,
     * vagy a WordPress (child) Téma mappájában a plugin nevével azonos mappában, azon belül pedig külön mappákban kell lenniük.
     * 
     * include_once(dirname(__file__) .DIRECTORY_SEPARATOR. 'templates.php');
     * $temp = new JTemplates($options);
     * echo $temp->prepareContent();
     *
     * @package Sportnaptár plugin  
     * @author InfoScope Kft.
     * @copyright Minden jog fenntartva!
     * @version 2020 v1.01
     * @access public
     */
    class JTemplates extends stdClass {
        // A plugin azonosítója
        private $ID = '';
        // Paraméterek, melyeket a sablonban be kell helyettesíteni
        private $options = null;
        // Hibaüzenetek tömbje
        private $errors = array();
        // Generált tartalom
        private $content = '';
        // A plugin elérési útja a fájlrendszerben
        private $plugin_path = '';
        // Az egyes sablonok elérési útja (CSS és JS beillesztéshez)
        private $template_sources  = array();
        // Számláló az egyes tömbökhöz
        private $counters = array();
        
        /**
         * JTemplates::__construct()
         * 
         * @param text $ID Egyedi azonosító, amelyet a HTML blokk megjelenítéséhez használ 
         * @param array $options A feldolgozandó adatok tömbben 
         * @return HTML a generált HTML tartalom
         */
        public function __construct($options = array()) {
            
            // Plugin elérési útja meghatározása
            $this->plugin_path = plugin_dir_path(dirname(__file__));
            
            // Azonosító meghatározása a plugin elérési útjából.
            $this->ID = explode(DIRECTORY_SEPARATOR, rtrim($this->plugin_path, DIRECTORY_SEPARATOR));
            $this->ID = array_pop($this->ID);
            
            // A kapott adatok áthelyezése a privát változóba
            $this->options = $options;
            
            // Alapértelmezett értékek hozzáadása
            $this->setDefaults();

            // Egyedi azonosító hozzárendelése a blokkhoz, mely a későbbeni javascript módosítást teszti lehetővé.
            if (!isset($this->options['token']) or (empty($this->options['token']))) {
                $this->options['token'] = $this->getToken();
            }

            // Ha nincs beállított sablon, akkor az alapértelmezett kerül beállításra
            if (!isset($this->options['template']) or empty($this->options['template'])) {
                $this->options['template_folder'] = '';
                $this->options['template_file'] = 'template.html';
            } else {

                $info = pathinfo($this->options['template']);
                if ($info['dirname'] == '.') {
                    $this->options['template_folder'] = $info['basename'] . DIRECTORY_SEPARATOR;
                    $this->options['template_file'] = 'template.html';
                } else {
                    $this->options['template_folder'] = $info['dirname'] . DIRECTORY_SEPARATOR;
                    $this->options['template_file'] = $info['basename'];
                    
                }
            }
            
            // Adatok előfeldolgozása
            $this->options = $this->preProcess($this->options);

        }

        /**
         * JTemplates::preProcess()
         * 
         * Az adatok előfeldolgozását végzi - módosítja az $options tömböt.
         * Akkor lehet fontos, ha pl. az egyik változó megléte esetén generálni kell egy másik változót is
         * 
         * @param array $options A módosítandó adat-tömb
         * @return void
         */
        function preProcess($options) {
            return $options;
        }

        /**
         * JTemplates::setDefaults()
         * 
         * Alapértelmezett változó értékek hozzáadása és ezáltal a sablonban használhatóság biztosítása
         * Ha nincs rá szükség, érdemes felülírni, hogy gyorsítsa a végrehajtást.
         * Ugyancsak felülírandó, ha saját alapértelmezett értékeket akarunk hozzáadni.
         * 
         * @return void
         */
        function setDefaults() {
            
            global $wp;
            
            $date_format = get_option( 'date_format' );
            $time_format = get_option( 'time_format' );
            
            // A honlap neve
            if (!isset($this->options['blog_name'])) {
                $this->options['blog_name'] = get_bloginfo('name');
            }
            // A honlap címoldala linkje
            if (!isset($this->options['home_url'])) {
                $this->options['home_url'] = esc_url( home_url( '/' ) );
            }
            // Az aktuális oldal linkje
            if (!isset($this->options['current_url'])) {
                $this->options['current_url'] = esc_url( home_url( $wp->request . '/') );
            }
            // Pillanatnyi dátum
            if (!isset($this->options['date'])) {
                $this->options['date'] = date_i18n( $date_format );
            }
            // Pillanatnyi idő
            if (!isset($this->options['time'])) {
                $this->options['time'] = date_i18n( $time_format );
            }
            // Pillanatnyi dátum és idő
            if (!isset($this->options['datetime'])) {
                $this->options['datetime'] = date_i18n( $date_format .' ' . $time_format);
            }
        }

        /**
         * JTemplates::getToken()
         * 
         * Egyedi azonosító generálása, mely ahhoz szükséges, hogy ha több oldalon kerül meghívásra a plugin, akkor 
         * a tartalmak külön azonosítót kapjanak és így manipulálhatóak legyenek
         * 
         * @return Egyedi azonosító
         */
        private function getToken() {
            // Egyedi azonosító generálása
            $chars = 'ABCDEFGHIJKLMNOPQURSTXYVWZ0123456789';
            $token = '';
            for ($i = 1; $i <= 10; $i++) {
            	$token .= substr($chars, rand(0, strlen($chars)-1), 1);
            }                
            return $token;
        }

        /**
         * JTemplates::getTemplate()
         * 
         * Megkeresi, hogy hol van a fájlrendszerben a használandó sablon és visszaadja azt
         * 
         * @param text $filename A betöltendő fájl neve
         * @param boolean $load_default Ha nem található, akkor betöltse-e a "default" sablonból
         * @return HTML sablon tartalma
         */
        function getTemplate($filename = '', $load_default = true) {
            
            if (empty($filename)) {
                $filename = $this->options['template_file'];
            }
            
            $template_source = '';
            
            if (file_exists(get_stylesheet_directory() .DIRECTORY_SEPARATOR. $this->ID .DIRECTORY_SEPARATOR. $this->options['template_folder'] . $filename)) {
                // Elsőként a (child) témában keresi
                $template = file_get_contents(get_stylesheet_directory() .DIRECTORY_SEPARATOR. $this->ID .DIRECTORY_SEPARATOR. $this->options['template_folder'] . $filename);
                $info = pathinfo(get_stylesheet_directory() .DIRECTORY_SEPARATOR. $this->ID .DIRECTORY_SEPARATOR. $this->options['template_folder'] . $filename);
                $template_source = $info['dirname'];
            } else {
                if (file_exists($this->plugin_path . 'templates' .DIRECTORY_SEPARATOR. $this->options['template_folder'] . $filename)) {
                    // Majd a plugin mappájában keresi
                    $template = file_get_contents($this->plugin_path . 'templates' .DIRECTORY_SEPARATOR. $this->options['template_folder'] . $filename);
                    $info = pathinfo($this->plugin_path . 'templates' .DIRECTORY_SEPARATOR. $this->options['template_folder'] . $filename);
                    $template_source = $this->plugin_path . 'templates' .DIRECTORY_SEPARATOR. $this->options['template_folder'];
                } else {
                    $this->errors[] = "\r\n" . '<!-- ['.$this->ID.']: ' . __('Template not available', '.$this->ID.') . ' ('.$this->options['template_folder'].$filename.') -->' . "\r\n";
                    if ($load_default and ($this->options['template'] != 'default')) {
                        // Ha a "default" sablon betöltendő
                        if (file_exists($this->plugin_path .DIRECTORY_SEPARATOR. 'templates' .DIRECTORY_SEPARATOR. 'default.html')) {
                            $template = file_get_contents($this->plugin_path . 'templates' .DIRECTORY_SEPARATOR. 'default.html');
                            $info = pathinfo($this->plugin_path . 'templates' .DIRECTORY_SEPARATOR. 'default.html');
                            $template_source = $this->plugin_path .DIRECTORY_SEPARATOR. 'templates';
                            $this->errors[] = "\r\n" . '<!-- ['.$this->ID.']: ' . __('Load default template', '.$this->ID.') . ' -->' . "\r\n";
                        } else {
                            return '';
                        }                    
                    } else {
                        return '';
                    }
                }
            }

            // Ha üres a sablon, akkor egy hibaüzenetet állítunk be.
            if (empty($template)) {
                $this->errors[] = "\r\n" . '<!-- ['.$this->ID.']: ' . $this->plugin_path . 'templates' .DIRECTORY_SEPARATOR. $this->options['template_folder'] . $filename.' -->' . "\r\n";
                $this->errors[] = "\r\n" . '<!-- ['.$this->ID.']: ' . __('Template is empty', '.$this->ID.') . ' ('.$this->options['template'].'/'.$filename.') -->' . "\r\n";                
            }
            
            // CSS és JS fájlok betöltéséhez a vizsgálandó mappák kijelölése
            // Ha az aktuális sablon mappája nincs a listában, felvesszük.
            if (!empty($template_source) and (!in_array($template_source, $this->template_sources))) {
                $this->template_sources[] = $template_source;   
            }

            // Get includes in template
            $template = $this->includeTemplate($template);

            return $template;
            
        }

        /**
         * JTemplates::includeTemplate()
         * 
         * Include more HTML files
         * 
         * @param mixed $template HTML content
         * @return HTML
         */
        function includeTemplate($template) {

            $includes = array();
            // Get the includes files list
            $regex = '/<!-- include (\s*.*?) -->/s';
            preg_match_all( $regex, $template, $matches);

            if (!empty($matches)) {
                foreach ($matches[1] as $file) {
                    if (!in_array($file, $includes)) {
                        $includes[] = $file;
                    }
                }
            }
            
            foreach ($includes as $file) {
                $tmpl = $this->getTemplate($file);    
                $tmpl = $this->renderContent($tmpl);   
                $template = str_replace('<!-- include '.$file.' -->', $tmpl, $template);         
            }
            
            return $template;
        }

        
        function getCSSandJS($path) {
            
            if (is_dir(get_stylesheet_directory() .DIRECTORY_SEPARATOR. $this->ID .DIRECTORY_SEPARATOR. $path)) {
                $this->template_sources[] = get_stylesheet_directory() .DIRECTORY_SEPARATOR. $this->ID .DIRECTORY_SEPARATOR. $path;
            } else {
                if (is_dir($this->plugin_path . 'templates' .DIRECTORY_SEPARATOR. $path)) {
                    $this->template_sources[] = $this->plugin_path . 'templates' .DIRECTORY_SEPARATOR. $path;                    
                }
            }           

            if (!empty($this->template_sources)) {
                $this->loadCSSandJS();
            }
            
        }


        /**
         * JTemplates::loadCSSandJS()
         * 
         * CSS és JS fájlok betöltése a mappákból
         * 
         * @return void
         */
        function loadCSSandJS() {
            
            $css = array();
            $js  = array();

            // Minden kijelölt mappát végignézünk
            foreach ($this->template_sources as $folder) {

                // Relatív útvonal kiszámítása ehhez a mappához
                $url = esc_url( home_url( '/' ) );
                $url = $url . str_replace(ABSPATH, '', $folder);

                $handle = @opendir($folder);
                if ($handle) {
                    while (false !== ($file = readdir($handle))) {
                        // Csak akkor vizsgálódunk, ha ez egy fájl
                        if (!is_dir($folder .DIRECTORY_SEPARATOR. $file)) {
                            $info = pathinfo($file);
                            // Csak a CSS és JS fájlokat dolgozzuk fel.
                            if (in_array($info['extension'], array('css', 'js'))) {
                                // Fájl hozzáadása a css fájlokhoz
                                if ($info['extension'] == 'css') {
                                    $css[] = $url .DIRECTORY_SEPARATOR. $file;
                                }
                                // Fájl hozzáadása a css fájlokhoz
                                if ($info['extension'] == 'js') {
                                    $js[] = $url .DIRECTORY_SEPARATOR. $file;
                                }
                            }
                        }
                    }
                }
                
            }

            // Ha vannak CSS fájlok, akkor azok mindegyikét betöltjük
            if (!empty($css)) {
                for ($i = 0; $i <= count($css)-1; $i++) {
                    wp_register_style( $this->ID.'-style-'.$this->options['template'].'-'.$i, $css[$i] );
                	wp_enqueue_style(  $this->ID.'-style-'.$this->options['template'].'-'.$i );
                }
            }
            
            // Ha vannak JS fájlok, akkor azok mindegyikét betöltjük
            if (!empty($js)) {
                for ($i = 0; $i <= count($js)-1; $i++) {
        			wp_register_script($this->ID.'-js-'.$this->options['template'].'-'.$i, $js[$i], array('jquery'));  
                    wp_enqueue_script($this->ID.'-js-'.$this->options['template'].'-'.$i);
                }
            }
            
        }

        /**
         * JTemplates::renderArray()
         * 
         * Egy tömböt renderel le
         * 
         * @param mixed $key A sablon nevét adó kulcs
         * @param mixed $data A behelyettesítendő adatok
         * @return void
         */
        private function renderArray($key, $data) {

            $template = $this->getTemplate($key.'.html');

            $arr = array();
            
            $this->counters[$key] = 0;
            
            foreach ($data as $item) {

                $temp = $template;
                $temp = $this->conditionalTemplate($temp, $item);
                
                $item = $this->preProcess($item);

                if (is_array($item) or is_object($item)) {                

                    $this->counters[$key]++;
                    foreach ($item as $k => $v) {
                        
                        // Ha ez egy objektum, akkor tömböt csinálunk belőle.
                        if (is_object($v)) { $v = (array) $v; }
                        
                        if (is_array($v)) {
                            // Ha ez egy tömb
                            $html = $this->renderArray($k, $v);
                            $temp = str_replace('<!-- ' . $k . ' -->', $html, $temp);
                        } else {
                            $temp = str_replace('<!-- ' .$key. '_counter -->', $this->counters[$key], $temp);
                            $temp = str_replace('<!-- ' .$k. ' -->', $this->modifyValue($k, $v), $temp);
                        }
                    }
                } else {

                    $temp = str_replace('<!-- ' .$key. ' -->', $this->modifyValue($key, $item), $temp);

                    $keys = array_keys($data);
                    foreach ($keys as $k) {
                        $temp = str_replace('<!-- ' .$k. ' -->', $this->modifyValue($k, $data[$k]), $temp);
                    } 
                }
                $arr[] = $temp;
            }

            return implode('', $arr);
        }

        /**
         * JTemplates::renderContent()
         * 
         * HTML tartalom renderelése, a változók behelyettesítése
         * 
         * @param mixed $template
         * @return void
         */
        private function renderContent($template) {
            
            // Ha üres a sablon, akkor üres a renderelt tartalom is
            if (empty($template)) { return ''; }

            // Feltételes formázásnak megfelelő sablon módosítás
            $template = $this->conditionalTemplate($template);

            foreach ($this->options as $key => $value) {
            // Adatok behelyettesítése

                // Ha ez egy objektum, akkor tömböt csinálunk belőle.
                if (is_object($value)) { $value = (array) $value; }

                if (is_array($value)) {
                    // Ha ez egy tömb, akkor azt más eljárással generáljuk
                    $html = $this->renderArray($key, $value);
                    $template = str_replace('<!-- ' . $key . ' -->', $html, $template);
                } else {
                    $template = str_replace('<!-- ' .$key. ' -->', $this->modifyValue($key, $value), $template);
                }
            }

            return $template;
        }

        /**
         * JTemplates::modifyValue()
         * 
         * A változó értékének módosítása - ez felülírandó a konkrét alkalmazás során, ha szükség van rá
         * 
         * @param $key A változó kulcsa 
         * @param $value A változó értéke 
         * @return $value Módosított érték
         */
        function modifyValue($key, $value) {
            /*
            if (!empty($key)) {
                switch ($key) { 
                	case 'timestamp':
                	   $value = date_i18n(get_option( 'date_format' ).' '.get_option( 'time_format' ), $value);
                       break;
                
                	default :
                }                
            }
            */
            return $value;
        }


        /**
         * JTemplates::conditionalTemplate()
         * 
         * A sablonban deklarált feltételrendszernek megfelelően formázza a sablont. (Törli a felesleges értékeket.)
         * 
         * @param HTML $template Az aktuálisan használt sablon HTML tartalma
         * @return void
         */
        public function conditionalTemplate($template, $data = array()) {
            
            // Ha nem kap adattartalmat, akkor az osztály adatait tartalmazza
            if (empty($data)) {
                $data = $this->options;
            }

            // Ha ez egy objektum, akkor tömböt csinálunk belőle.
            if (is_object($data)) { $data = (array) $data; }
            
            // Feltételekhez használt kapcsolók tömbje
            $conditionals = array();
            $regex = '/<!-- if (\s*.*?) -->/s';
            preg_match_all( $regex, $template, $matches);
            if (!empty($matches)) {
                foreach ($matches[1] as $name) {
                    if (!in_array($name, $conditionals)) {
                        $conditionals[] = $name;
                    }
                }
            }
            
            // Minden feltételhez kapcsolt változóra vizsgáljuk a sablont
            foreach ($conditionals as $conditional) {

                $regex = '/<!-- if '.$conditional.' -->(\s*.*?)<!-- endif '.$conditional.' -->/s';
                preg_match_all( $regex, $template, $matches);
                
                if (!empty($matches)) {
                    for ($i = 0; $i <= count($matches[0])-1; $i++) {
                        $content = explode('<!-- else '.$conditional.' -->', $matches[1][$i]);
                        if (isset($data[$conditional]) and !empty($data[$conditional])) {
                            $template = str_replace($matches[0][$i], $content[0], $template);
                        } else {
                            if (isset($content[1])) {
                                $template = str_replace($matches[0][$i], $content[1], $template);
                            } else {
                                $template = str_replace($matches[0][$i], '', $template);
                            }
                        }
                    }
                }
            } 
            return $template;           
        }

        /**
         * JTemplates::prepareContent()
         * 
         * A generált HTML tartalom visszaadása
         * 
         * @return HTML
         */
        function prepareContent() {
            
            // Betölti a sablont a megfelelő helyről 
            $template = $this->getTemplate($this->options['template_file']);
            
            // CSS és JS jájlok betöltése
            $this->loadCSSandJS();

            // HTML tartalom renderelése
            $this->content = $this->renderContent($template);

            $html = "\r\n";
            if (WP_DEBUG) {
                $html .= implode('', $this->errors);
            } 
            $html .= $this->content;
            
            return $html;
        }

    }